/* ----------------------------------------------------------
                      code_snippets.c

   Hier finden sich Funktionen, fuer die es sich nicht lohnt
   .c / .h Dateien anzulegen. Diese Datei kann auch nicht
   gelinkt werden, sie dient dazu, einzelne Funktionen per
   Copy & Paste in andere Projekte einzufuegen

   MCU      :  Attiny44
   Takt     :  8 MHz intern

   Fuses    :  Lo:0xE2    Hi:0xDF

   14.08.2018 R. Seelig
   ---------------------------------------------------------- */
/*

/* ##########################################################
                      SPI (nur lesend)
   ########################################################## */

  #define hardware_spi          1        // 0 : SPI ueber Bitbanging realisieren, hier
                                         //     duerfen dann die Anschlusspins veraendert werden
                                         // 1 : SPI ueber USI-Hardware realisieren,
                                         //     Anschlusspins fuer Mosi, Miso, Clk duerfen
                                         //     NICHT veraendert werden


  #define sw_mosiinit()         PA5_output_init()
  #define sw_sckinit()          PA4_output_init()

  #define mosi_set()            ( PA5_set() )
  #define mosi_clr()            ( PA5_clr() )
  #define sck_set()             ( PA4_set() )
  #define sck_clr()             ( PA4_clr() )

#if (hardware_spi == 0)

  /* -----------------------------------------------------
                             spi_out

          Byte ueber Software SPI senden / empfangen
          data ==> zu sendendes Datum
     ----------------------------------------------------- */
  void spi_out(uint8_t value)
  {
    char a;

    for (a= 0; a< 8; a++)
    {
      if((value & 0x80)> 0) { mosi_set(); }
                       else { mosi_clr(); }
      // Taktimpuls erzeugen
      sck_set();
      sck_clr();
      value= value << 1;
    }
  }
#endif

#if (hardware_spi == 1)

  /* -------------------------------------------------------------
                             spi_out

        Byte ueber USI Hardware senden / empfangen
        data ==> zu sendendes Datum
     ------------------------------------------------------------- */
  unsigned char spi_out(uint8_t data)
  {
    volatile uint8_t r;

    USIDR = data;
    USISR = (1 << USIOIF);

    while ( (USISR & (1 << USIOIF)) == 0 )
    {
      USICR = (1 << USIWM0) | (1 << USICS1) | (1 << USICLK) | (1 << USITC);
    }
    return USIDR;
  }
#endif

/* ----------------------------------------------------------
                         ENDE SPI
   ---------------------------------------------------------- */


/* ##########################################################
                         Timer0 compare match
   ########################################################## */
/* -------------------------------------------------------
         Interruptvektor, Timer0 compare match

         Intervall wird durch timer0_init bestimmt
   ------------------------------------------------------- */
ISR (TIM0_COMPA_vect)
{
  // hier Code einfuegen, der mittels Intervall abge-
  // arbeitet werden soll.

  TCNT0= 0;
}

/* -------------------------------------------------------
                        timer0_init

     Initialisiert den Timer0 (8-Bit) als Comparetimer.

     Uebergabe:
        prescale  : Teilerfaktor von F_CPU mit der der
                    Timer getaktet wird.
        compvalue : Vergleichswert, bei dem ein Interrupt
                    ausgeloest wird
   ------------------------------------------------------- */
void timer0_init(uint8_t prescale, uint8_t compvalue)
{
  /*
  TCCR0B : Timer/Counter Controllregister B
    u.a. verantwortlich fuer das Einstellen der Taktversorgung des Zaehlers.
    Bits sind: CS02..CS00
       1 : keine Teilung, 2 : div(8), 3 : div(64), 4 : div(256), 5 : div(1024)
  OCR0A : Output Compare Register, eines von zwei Vergleichsregistern
    erreicht der Zaehler TCNT0 den Wert von OCR0A kann ein Interrupt ausgeloest
    werden (wenn dieses im TIMSK0 Register aktiviert ist)
  TIMSK0 : Timer Interrupt Mask Register
    bestimmt ob und bei welchem Ereignis ein Interrupt ausgeloest werden soll.
    OCIE0A (Bit1) loest bei Gleichheit von OCR0A und TCNT0 einen Interrupt aus.
    Interruptvektor fuer OCIE0A ist:   ISR (TIMER0_COMPA_vect)   (fuer ATtiny44)

    Bsp.:
       F_CPU = 8 MHz, prescaler 1024 = 7812,5 Hz = 128uS Taktzeit Zaehler
       OCR0A = 16: nach 128uS * 16 = 2,048 mS wird ein Interupt ausgeloest (bei
               Startwert TCNT0 = 0).
  */
  TCCR0B = prescale;
  OCR0A = compvalue;
  TCNT0 = 0;                   // Zaehler zuruecksetzen

  TIMSK0 = 1 << OCIE0A;        // if OCR0A == TCNT0 dann Interrupt
  sei();                       // Interrupts grundsaetzlich an

}

/* ----------------------------------------------------------
                         ENDE Timer0 compare match
   ---------------------------------------------------------- */

/* ##########################################################
                         PWM (Timer1)
   ########################################################## */
/* -----------------------------------------------------
                       pwm_init

    initialisiert einen PWM Generator mit angegebenem
    PWM Ausgangspin (bei ATtiny44 ist das PA6

    Uebergabe:
       Der Grundtakt des internen PWM Zaehlers ist
       vom CPU-Takt abhaengig. Dieser Takt kann
       geteilt zugefuehrt werden

       prescale:
         1 : keine Taktteilung     2 : F_CPU / 8
         3 : F_CPU / 64            4 : F_CPU / 256
         5 : F_CPU / 1024
   ----------------------------------------------------- */
void pwm_init(uint8_t prescale)
{
   pwmdir |= (1 << pwmout);                   // OC1A .. PA6 als PWM-Ausgang

   // WGM10= 0 ; WGM11 WGM12, WGM13 = 1 ===> Modus 14 (Fast PWM)
   // Prescaler wird durch CS10, CS11, CS12 gesetzt

   TCCR1A = (1<<COM1A1) | (1<<WGM11);
   TCCR1B = (1<<WGM12) | (1<<WGM13) | prescale;
}

/* -----------------------------------------------------
                          setpwm

     setzt die Parameter fuer Gesamtlaenge und Pulsdauer
     einer Pulsweitenmodulation.

     Uebergabe:
       tim1 : Gesamtdauer (in Digits)
       tim2 : Pulsdauer (in Digits)

       tim1 / tim2 ist somit das Verhaeltnis von Ge-
       samtdauer zu Pulsdauer. Wie lange die Gesamt-
       dauer ist (65536 Digits) ist von pwm_init ab-
       haengig.
   ----------------------------------------------------- */
void setpwm(uint16_t tim1, uint16_t tim2)
{
  // tim1 ==> Gesamtdauer
  // tim2 ==> Pulsdauer

  ICR1H = (tim1 >> 8);
  ICR1L = (uint8_t) tim1 & 0x00ff;
  OCR1AH = (tim2 >> 8);
  OCR1AL = (uint8_t) tim2 & 0x00ff;
}
/* ----------------------------------------------------------
                         ENDE PWM
   ---------------------------------------------------------- */
